----------------------------------------------------------------------------
-- @Author: ViperGTS96------------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------

rearviewMirror = {};
rearviewMirror.dir = g_currentModDirectory;
local modDescFile = loadXMLFile("modDesc", rearviewMirror.dir .. "modDesc.xml");
rearviewMirror.title = getXMLString(modDescFile, "modDesc.title.en");
rearviewMirror.author = getXMLString(modDescFile, "modDesc.author");
rearviewMirror.version = getXMLString(modDescFile, "modDesc.version");
rearviewMirror.isVisible = false;
rearviewMirror.startVisible = Utils.getNoNil(getXMLBool(modDescFile, "modDesc.startVisible"),false);
delete(modDescFile);
rearviewMirror.includeHorse = false;
rearviewMirror.positionSmoothing = 0;
rearviewMirror.lookAtSmoothing = 0;
rearviewMirror.positionSmoothingIn = 0;
rearviewMirror.lookAtSmoothingIn = 0;

-----------------------------------
print("Load mod: "..rearviewMirror.title.." : v"..rearviewMirror.version.." by "..rearviewMirror.author);
-----------------------------------

function rearviewMirror:loadMap(savegame)

	rearviewMirror.events = {};
    Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, rearviewMirror.registerActionEvents);
	Player.removeActionEvents = Utils.appendedFunction(Player.removeActionEvents, rearviewMirror.removeActionEvents);
    Enterable.onRegisterActionEvents = Utils.appendedFunction(Enterable.onRegisterActionEvents, rearviewMirror.registerActionEvents);

	Player.update = Utils.appendedFunction(rearviewMirror.updatePlayer, Player.update);
	Enterable.onPostUpdate = Utils.appendedFunction(rearviewMirror.updateVehicle, Enterable.onPostUpdate);

	if rearviewMirror.startVisible then
		rearviewMirror.isVisible = true;
	end;

end;


function rearviewMirror:updatePlayer(dt)

	if self.cameraNode ~= nil then	
		if self.rearViewMirror == nil then
			self.rearViewMirror = loadI3DFile(rearviewMirror.dir.."mirror.i3d", false, false, false);
			link(self.cameraNode, self.rearViewMirror);
			setTranslation(self.rearViewMirror,0,0.5,-1.0);
		else
			local camVisibility = getVisibility(self.rearViewMirror);
			if camVisibility ~= rearviewMirror.isVisible then
				setVisibility(self.rearViewMirror, (not camVisibility));
			end;
		end;
	end;

end;

function rearviewMirror:updateVehicle()

    local spec = self.spec_enterable
    if spec.isEntered then
		if spec.activeCamera ~= nil then
			local camera = spec.activeCamera;
			if camera.rearViewMirror == nil then
				if camera.isInside ~= nil and camera.isInside then
					camera.rearViewMirror = loadI3DFile(rearviewMirror.dir.."mirror.i3d", false, false, false);		
					link(camera.cameraPositionNode, camera.rearViewMirror);
					setScale(camera.rearViewMirror,1.5,1.0,1.0);
					setTranslation(camera.rearViewMirror,0.0,0.14,-0.22);
				else
					if self.spec_rideable == nil or rearviewMirror.includeHorse then -- not a horse
						camera.rearViewMirror = loadI3DFile(rearviewMirror.dir.."mirror.i3d", false, false, false);
						link(camera.cameraPositionNode, camera.rearViewMirror);
						setTranslation(camera.rearViewMirror,0,0.44,-1.0);
						rearviewMirror.positionSmoothing = camera.positionSmoothingParameter;
						rearviewMirror.lookAtSmoothing = camera.lookAtSmoothingParameter;
						camera.positionSmoothingParameter = 1.0;
						camera.lookAtSmoothingParameter = 0.2;
					end;
				end;
			else
				local camVisibility = getVisibility(camera.rearViewMirror);
				if camVisibility ~= rearviewMirror.isVisible then
					if camVisibility then
						setVisibility(camera.rearViewMirror, false);
						if not camera.isInside then
							camera.positionSmoothingParameter = rearviewMirror.positionSmoothing;
							camera.lookAtSmoothingParameter = rearviewMirror.lookAtSmoothing;
						end;
					else
						setVisibility(camera.rearViewMirror, true);
						if not camera.isInside then
							camera.positionSmoothingParameter = 1.0;
							camera.lookAtSmoothingParameter = 0.2;
						end;
					end;
				end;
			end;
			if spec.cameras ~= nil then -- Remove inactive mirrors
				for _, cam in pairs(spec.cameras) do
					if camera ~= cam then
						if cam.rearViewMirror ~= nil then
							unlink(cam.rearViewMirror);
							cam.rearViewMirror = nil;
						end;
					end;
				end;
			end;
		end;
	else
		if spec.cameras ~= nil then -- Remove all mirrors on Exit
			for _, camera in pairs(spec.cameras) do
				if camera.rearViewMirror ~= nil then
					unlink(camera.rearViewMirror);
					camera.rearViewMirror = nil;
				end;
			end;
		end;
	end;

end;

addModEventListener(rearviewMirror);

function rearviewMirror:removeActionEvents()
	rearviewMirror.events = {}; --needed?
end;

function rearviewMirror:registerActionEvents()
	if self.spec_rideable == nil or rearviewMirror.includeHorse then
		local result, eventName = InputBinding.registerActionEvent(g_inputBinding, 'TOGGLEVISIBILITY', self, rearviewMirror.toggleVisibility, false, true, false, true)
		if result then
			table.insert(rearviewMirror.events, eventName);
			g_inputBinding.events[eventName].displayIsVisible = true;
		end;
    end;
end;

function rearviewMirror:toggleVisibility(actionName, keyStatus, arg3, arg4, arg5)
	if actionName == "TOGGLEVISIBILITY" then
		rearviewMirror.isVisible = not rearviewMirror.isVisible;
	end;
end;